from typing import List, Optional
from redapy.base import TorrentGroup, Torrent, urls, BaseAuth


class Search():
    def __init__(self):
        self.sess = BaseAuth()
        self.url = urls["base_url"] + urls["torrent_search"]

    # TODO searchstr can be none // all but page kwargs, dejsonify
    # -> List[TorrentGroup]:
    def search_torrent(self, search_str: str, page: Optional[int] = 1, **kwargs) -> List[TorrentGroup]:
        params = {
            "searchstr": search_str,
            "page": page,
            **kwargs
        }
        response = self.sess.session.get(self.url, params=params)
        r_json = response.json()
        results = r_json["response"]["results"]

        groups: List[TorrentGroup] = []

        for i in results:
            torrents = i["torrents"]
            torr_list: List[Torrent] = []
            for j in torrents:
                torrent = Torrent(
                    j["torrentId"],
                    [x for x in j["artists"]],  # needs proper impl
                    j["remastered"],
                    j["remasterYear"],
                    j["remasterCatalogueNumber"],
                    j["remasterTitle"],
                    j["media"],
                    j["encoding"],
                    j["format"],
                    j["hasLog"],
                    j["logScore"],
                    j["hasCue"],
                    j["scene"],
                    j["vanityHouse"],
                    j["fileCount"],
                    j["time"],
                    j["size"],
                    j["snatches"],
                    j["seeders"],
                    j["leechers"],
                    j["isFreeleech"],
                    j["isFreeload"],
                    j["isPersonalFreeleech"],
                    j["canUseToken"],
                    j["hasSnatched"]

                )
                torr_list.append(torrent)
            group = TorrentGroup(
                i["groupId"],
                i["groupName"],
                i["artist"],
                [x for x in i["tags"]],
                bool(i["bookmarked"]),
                bool(i["vanityHouse"]),
                i["groupYear"],
                i["releaseType"],
                torr_list
            )
            groups.append(group)

        return groups



from base import BaseAuth, urls


class Index():
    def __init__(self):
        self.url: str = urls["base_url"] + urls["index"]

    def get_index(self):
        s = BaseAuth()
        r = s.session.get(self.url)
        print(r.content)



from redapy.base import BaseAuth, TorrentDownload, urls, Failure
from typing import Optional, Union
import os


class Download():
    def __init__(self) -> None:
        self.sess = BaseAuth()

    def get_torrent_data(self, id: int) -> TorrentDownload:
        self.url_details: str = urls["base_url"] + urls["torrent_details"]

        r = self.sess.session.get(self.url_details + f'id={id}')
        resp_json = r.json()

        artists = [i["name"]
                   for i in resp_json["response"]["group"]["musicInfo"]["artists"]]
        name = resp_json["response"]["group"]["name"]
        media = resp_json["response"]["torrent"]["media"]
        format = resp_json["response"]["torrent"]["format"]
        encoding = resp_json["response"]["torrent"]["encoding"]
        torrent = TorrentDownload(artists, name, id, media, format, encoding)
        return torrent

    def download_torrent(self, id: int, target_dir: Optional[str] = os.getcwd(), token: Optional[Union[int, bool]] = 0):
        self.url_dl: str = urls["base_url"] + urls["torrent_dl"]

        if type(token) == bool:
            token = int(token)

        r = self.sess.session.get(
            self.url_dl + f'id={id}&usetoken={token}', stream=True)
        r_cont_b = bytes(r.content)

        torrent = self.get_torrent_data(id)
        artists = ', '.join(torrent.artists)
        filename = f'{artists} - {torrent.name} [{torrent.media} {torrent.format} {torrent.encoding}].torrent'
        filepath: str = os.path.join(target_dir, filename)

        try:
            with open(filepath, 'wb') as f:
                f.write(r_cont_b)
        except IOError:
            return Failure(f'Cannot open file for writing {filename}')



from typing import List
from base import BaseAuth, URLs, Request, mappings_release, urls


class Artist():
    def __init__(self):
        # urls = URLs()
        self.url: str = urls["base_url"] + urls["artist"]
        self.sess = BaseAuth()

    def artist_by_id(self, id: int):
        r = self.sess.session.get(self.url + f'id={id}')
        resp_json = r.json()
        return resp_json

    # to implement further
    # def artist_by_name(self, name: str):
    #     name = name.replace(' ', '+')
    #     r = self.sess.session.get(self.url + f'artistname={name}')
    #     return r.content

    def get_artist_torrent_ids(self, id: int, format: str = "", encoding: str = "") -> List[int]:
        artist_page: dict = self.artist_by_id(id)
        torrentgroups: List[dict] = artist_page["response"]["torrentgroup"]
        torrent_ids: List[int] = []
        # it somehow works but is inefficient af
        for i in torrentgroups:
            i = dict(i)
            if format != "":
                if encoding != "":
                    for j in i["torrent"]:
                        if j["format"] == format and j["encoding"] == encoding:
                            torrent_ids.append(j["id"])
                else:
                    for j in i["torrent"]:
                        if j["format"] == format:
                            torrent_ids.append(j["id"])
            else:
                for j in i["torrent"]:
                    torrent_ids.append(j["id"])
        return torrent_ids

    def get_request(self, id: int) -> List[Request]:
        artist_page: dict = self.artist_by_id(id)
        requests_list: List[dict] = artist_page["response"]["requests"]
        requests: List[Request] = []
        for request in requests_list:
            requests.append(Request(
                request["requestId"], request["title"],
                mappings_release[int(request["categoryId"])],
                request["bounty"])
            )
        return requests



from dataclasses import asdict
import os
from typing import List, Union
from redapy.base import BaseAuth, TorrentUpload, urls, mappings_categories, mappings_release, mappings_artist_importance, Failure


# No Log support for now

class Upload():
    def __init__(self):
        self.sess = BaseAuth()
        self.url: str = urls["base_url"] + urls["torrent_up"]

    def prepare_torrent(self, category: Union[str, int], artists: List[str], importance: Union[List[str], List[int]],
                        title: str, year: int, release_type: Union[str, int],
                        format: str, bitrate: str, media: str, tags: List[str],
                        image: str, album_desc: str, torrent_desc: str, **kwargs):
        if type(category) == str:
            category: int = [
                i for i in mappings_categories if mappings_categories[i] == category][0]

        if type(release_type) == str:
            release_type: int = [
                i for i in mappings_release if mappings_release[i] == release_type][0]

        if type(importance[0]) == str:
            importance: List[int] = [[
                j for j in mappings_artist_importance if mappings_artist_importance[j] == i][0] for i in importance]
        try:
            if not kwargs:
                remaster_year = year
                torrent = TorrentUpload(category, artists, importance, title, year, release_type, format,
                                        bitrate, media, tags, image, album_desc, torrent_desc, remaster_year)
            elif kwargs["remaster_year"] is None:
                remaster_year = year
                torrent = TorrentUpload(category, artists, importance, title, year, release_type, format,
                                        bitrate, media, tags, image, album_desc, torrent_desc, remaster_year, **kwargs)
            else:
                torrent = TorrentUpload(category, artists, importance, title, year, release_type, format,
                                        bitrate, media, tags, image, album_desc, torrent_desc, **kwargs)
            return torrent
        except KeyError as e:
            return e

    def upload_torrent(self, filepath: str, filename: str, torrent: TorrentUpload):
        file = {
            "file_input": (filename, open(os.path.join(filepath, filename), 'rb'), 'application/x-bittorrent')
        }
        arguments = asdict(torrent)
        r = self.sess.session.post(self.url, data=arguments, files=file)
        resp = r.json()
        if resp["status"] == "success":
            return resp["response"]["torrentid"]
        else:
            return Failure("Failed to upload a torrent")

    # TODO report for LMA/LWA // no API endpoint in wiki // Low priority

    def report(self):
        pass



from dataclasses import dataclass
import requests
from dotenv import load_dotenv
import os
from typing import List, Dict, Optional, Union

mappings_release: Dict[int, str] = {
    1: "Album",
    3: "Soundtrack",
    5: "EP",
    6: "Anthology",
    7: "Compilation",
    9: "Single",
    11: "Live album",
    13: "Remix",
    14: "Bootleg",
    15: "Interview",
    16: "Mixtape",
    17: "Demo",
    18: "Concert Recording",
    19: "DJ Mix",
    21: "Unknown"
}

mappings_categories: Dict[int, str] = {
    0: "Music",
    1: "Applications",
    2: "E-Books",
    3: "Audiobooks",
    4: "E-Learning Videos",
    5: "Comedy",
    6: "Comics"
}

mappings_artist: Dict[int, str] = {
    1021: "Produced By",
    1022: "Composition",
    1023: "Remixed By",
    1024: "Guest Appearance"
}

mappings_artist_importance: Dict[int, str] = {
    1: "Main",
    2: "Guest",
    3: "Composer",
    4: "Conductor",
    5: "DJ / Compiler",
    6: "Remixer",
    7: "Producer",
}


@dataclass
class Request():
    id: int
    title: str
    category: int  # Album, EP ...
    bounty: int  # in bytes


@dataclass
class Artist():
    id: int
    name: str
    aliasid: int


@dataclass
class Torrent():
    id: int
    artists: List[Artist]
    remastered: bool
    remaster_year: int
    remaster_cat_no: str
    remaster_title: str
    media: str
    encoding: str
    format: str
    has_log: bool
    log_score: int
    has_cue: bool
    scene: bool
    vanity_house: bool
    file_count: int
    time_uploaded: str
    size: int  # in bytes
    snatches: int
    seeders: int
    leechers: int
    is_freeleech: bool
    is_freeload: bool
    is_personal_freeleech: bool
    can_use_token: bool
    has_snatched: bool


@dataclass
class TorrentGroup():
    group_id: int
    group_name: str
    artist: str
    tags: List[str]
    bookmarked: bool
    vanity_house: bool
    group_year: int
    release_type: Union[str, int]
    torrents: List[Torrent]


@dataclass
class TorrentDownload():
    artists: List[str]
    name: str
    id: int
    media: str
    format: str
    encoding: str


@dataclass
class TorrentUpload():
    category: int
    artists: List[str]
    importance: List[int]
    title: str
    year: int
    releasetype: int
    format: str
    bitrate: str
    media: str
    tags: str
    image: str
    album_desc: str
    desc: str
    remaster_year: Optional[int] = None  # if not provided = year
    unknown: Optional[bool] = None
    groupid: Optional[int] = None
    requestid: Optional[int] = None
    release_desc: Optional[str] = None
    vanity_house: bool = None
    other_bitrate: Optional[str] = None
    vbr: Optional[bool] = None
    remaster_title: Optional[str] = None
    remaster_record_label: Optional[str] = None
    remaster_catalogue_number: Optional[str] = None
    scene: Optional[bool] = None


@dataclass
class BookmarkedTorrent():
    group_id: int


class BaseAuth():
    def __init__(self):
        load_dotenv()
        self.session = requests.Session()
        self.session.headers.update({"Authorization": os.getenv('APIKEY')})


urls: Dict[str, str] = {
    "base_url": "https://redacted.ch/",
    "index": "ajax.php?action=index",
    "user_profile": "ajax.php?action=user",
    "inbox": "ajax.php?action=inbox",
    "conversation": "ajax.php?action=inbox&type=viewconv",
    "send_pm": "ajax.php?action=send_pm",
    "top10": "ajax.php?action=top10",
    "user_search": "ajax.php?action=usersearch",
    "bookmarks": "ajax.php?action=bookmarks&type=",
    "subs": "ajax.php?action=subscriptions",
    "forum_category": "ajax.php?action=forum&type=main",
    "forum_view": "ajax.php?action=forum&type=main",
    "thread_view": "ajax.php?action=forum&type=viewthread&threadid=",
    "artist": "ajax.php?action=artist&",
    "torrent_search": "ajax.php?action=browse&",
    "torrent_dl": "ajax.php?action=download&",
    "torrent_details": "ajax.php?action=torrent&",
    "torrent_up": "ajax.php?action=upload"
}


class Failure(Exception):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['redapy', 'redapy.forum', 'redapy.torrent', 'redapy.user', 'redapy.utils']

package_data = \
{'': ['*']}

install_requires = \
['python-dotenv>=0.20.0,<0.21.0', 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'redapy',
    'version': '0.1',
    'description': 'API wrapper module for redacted.ch',
    'long_description': None,
    'author': 'nagqu',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
